@@ -35,19 +35,20 @@ DATADIR = '@datadir@'
 
 NAME = 'SoundConverter'
 VERSION = '@version@'
-print '%s %s' % (NAME, VERSION)
+print( '%s %s' % (NAME, VERSION) )
 
 GLADEFILE = '@datadir@/soundconverter/soundconverter.glade'
 
 PACKAGE = NAME.lower()
 try: 
-    gettext.bindtextdomain(PACKAGE,'@datadir@/locale')
     locale.setlocale(locale.LC_ALL,'')
+    locale.bindtextdomain(PACKAGE,'@datadir@/locale')
+    gettext.bindtextdomain(PACKAGE,'@datadir@/locale')
     gettext.textdomain(PACKAGE)
     gettext.install(PACKAGE,localedir='@datadir@/locale',unicode=1)
     #from gettext import gettext as _
 except locale.Error:
-    print '  cannot use system locale.'
+    print('  cannot use system locale.')
     locale.setlocale(locale.LC_ALL,'C')
     gettext.textdomain(PACKAGE)
     gettext.install(PACKAGE,localedir='@datadir@/locale',unicode=1)
@@ -72,8 +73,8 @@ def _check_libs():
         import gobject
         gobject.threads_init()
         import gnomevfs
-    except ImportError:
-        print '%s needs pygtk and gnome-python >= 2.24!' % NAME
+    except ImportError, error :
+        print('%s needs pygtk and gnome-python >= 2.24! (Error: "%s")' % (NAME, error))
         sys.exit(1)
     except:
         pass
@@ -83,11 +84,11 @@ def _check_libs():
         pygst.require('0.10')
         import gst
     except ImportError:
-        print '%s needs python-gstreamer 0.10!' % NAME
+        print('%s needs python-gstreamer 0.10!' % NAME)
         sys.exit(1)
 
-    print '  using Gstreamer version: %s' % (
-            '.'.join([str(s) for s in gst.gst_version]))
+    print( '  using Gstreamer version: %s' % (
+            '.'.join([str(s) for s in gst.gst_version])) )
 
 
 def check_mime_type(mime):
@@ -95,11 +96,11 @@ def check_mime_type(mime):
         'mp3': 'audio/mpeg', 'aac': 'audio/x-m4a'}
     mime = types.get(mime, mime)
     if mime not in types.values():
-        print 'Cannot use "%s" mime type.' % mime
+        print('Cannot use "%s" mime type.' % mime)
         print 'Supported shortcuts and mime types:',
         for k,v in sorted(types.iteritems()):
             print '%s %s' % (k,v),
-        print
+        print()
         raise SystemExit
     return mime
 
@@ -165,11 +166,17 @@ settings['cli-output-type'] = check_mime_type(settings['cli-output-type'])
 
 _check_libs()
 
-print '  using %d thread(s)' % settings['jobs']
+import gtk
+import gtk.glade
+gtk.glade.bindtextdomain(PACKAGE, '@datadir@/locale')
+gtk.glade.textdomain(PACKAGE)
+
+print('  using %d thread(s)' % settings['jobs'])
 
 from soundconverter.batch import cli_convert_main
 from soundconverter.batch import cli_tags_main
 from soundconverter.fileoperations import filename_to_uri
+
 files = map(filename_to_uri, files)
 
 try:
@@ -181,10 +188,12 @@ except:
 if settings['mode'] == 'gui':
     gui_main(NAME, VERSION, GLADEFILE, files)
 elif settings['mode'] == 'tags':
-    if not files: print 'nothing to do...'
+    if not files:
+        print('nothing to do...')
     cli_tags_main(files)
 else:
-    if not files: print 'nothing to do...'
+    if not files:
+        print('nothing to do...')
     cli_convert_main(files)
 
 
