@@ -26,18 +26,20 @@ from gettext import gettext as _
 
 import gobject
 import gst
+import gst.pbutils
 import gnomevfs
 import gconf
 
 from fileoperations import vfs_encode_filename, file_encode_filename
 from fileoperations import unquote_filename, vfs_makedirs, vfs_unlink
+from fileoperations import vfs_rename
 from fileoperations import vfs_exists
+from fileoperations import beautify_uri
 from fileoperations import use_gnomevfs
 from task import BackgroundTask
 from queue import TaskQueue
 from utils import debug, log
 from settings import mime_whitelist, filename_blacklist
-from error import SoundConverterException
 from error import show_error
 try:
     from notify import notification
@@ -47,6 +49,19 @@ except:
         
 from fnmatch import fnmatch
 
+import time
+import gtk
+def gtk_iteration():
+    while gtk.events_pending():
+        gtk.main_iteration(False)
+
+
+def gtk_sleep(duration):
+    start = time.time()
+    while time.time() < start + duration:
+        time.sleep(0.010)
+        gtk_iteration()
+
 import gconf
 # load gstreamer audio profiles
 _GCONF_PROFILE_PATH = "/system/gstreamer/0.10/audio/profiles/"
@@ -77,7 +92,7 @@ for name in profiles:
 required_elements = ('decodebin', 'fakesink', 'audioconvert', 'typefind', 'audiorate')
 for element in required_elements:
     if not gst.element_factory_find(element):
-        print "required gstreamer element \'%s\' not found." % element
+        print("required gstreamer element \'%s\' not found." % element)
         sys.exit(1)
 
 if gst.element_factory_find('giosrc'):
@@ -85,19 +100,21 @@ if gst.element_factory_find('giosrc'):
     gstreamer_sink = 'giosink'
     encode_filename = vfs_encode_filename
     use_gnomevfs = True
-    print '  using gio'
+    print('  using gio')
 elif gst.element_factory_find('gnomevfssrc'):
     gstreamer_source = 'gnomevfssrc'
     gstreamer_sink = 'gnomevfssink'
     encode_filename = vfs_encode_filename
     use_gnomevfs = True
-    print '  using deprecated gnomevfssrc'
+    print('  using deprecated gnomevfssrc')
 else:
     gstreamer_source = 'filesrc'
     gstreamer_sink = 'filesink'
     encode_filename = file_encode_filename
-    print '  not using gnomevfssrc, look for a gnomevfs gstreamer package.'
+    print('  not using gnomevfssrc, look for a gnomevfs gstreamer package.')
 
+# used to dismiss codec installation if the user already canceled it
+user_canceled_codec_installation = False
 
 encoders = (
     ('flacenc',   'FLAC'),
@@ -141,20 +158,21 @@ class Pipeline(BackgroundTask):
         self.eos = False
         self.error = None
         self.connected_signals = []
-        self.aborted = False
 
     def started(self):
         self.play()
 
-    def finished(self):
+    def cleanup(self):
         for element, sid in self.connected_signals:
             element.disconnect(sid)
         self.connected_signals = []
         self.stop_pipeline()
 
-    def abort(self):
-        self.aborted = True
-        self.finished()
+    def aborted(self):
+        self.cleanup()
+
+    def finished(self):
+        self.cleanup()
 
     def add_command(self, command):
         self.command.append(command)
@@ -175,21 +193,27 @@ class Pipeline(BackgroundTask):
     def found_tag(self, decoder, something, taglist):
         pass
 
+    def restart(self):
+        self.parsed = False
+        self.duration = None
+        self.finished()
+        if vfs_exists(self.output_filename):
+            vfs_unlink(self.output_filename)
+        self.play()
+
     def install_plugin_cb(self, result):
         if result in (gst.pbutils.INSTALL_PLUGINS_SUCCESS,
                       gst.pbutils.INSTALL_PLUGINS_PARTIAL_SUCCESS):
             gst.update_registry()
-            self.parsed = False
-            self.duration = None
-            self.finished()
-            vfs_unlink(self.output_filename)
-            self.play()
+            self.restart()
+            return
+        if result == gst.pbutils.INSTALL_PLUGINS_USER_ABORT:
+            self.error = _('Plugin installation aborted.')
+            global user_canceled_codec_installation
+            user_canceled_codec_installation = True
+            self.done()
             return
         self.done()
-        if result == gst.pbutils.INSTALL_PLUGINS_USER_ABORT:
-            show_error(_('Plugin installation aborted.'))
-            return
-
         show_error('Error', 'failed to install plugins: %s' % gobject.markup_escape_text(str(result)))
 
     def on_error(self, error):
@@ -200,22 +224,28 @@ class Pipeline(BackgroundTask):
         t = message.type
         import gst
         if t == gst.MESSAGE_ERROR:
-            error, debug = message.parse_error()
+            error, _ = message.parse_error()
             self.eos = True
+            self.error = error
             self.on_error(error)
             self.done()
-
-        elif t == gst.MESSAGE_ELEMENT:
-            st = message.structure
-            if st and st.get_name().startswith('missing-'):
-                self.pipeline.set_state(gst.STATE_NULL)
-                if gst.pygst_version >= (0, 10, 10):
-                    import gst.pbutils
-                    detail = gst.pbutils.\
-                        missing_plugin_message_get_installer_detail(message)
-                    ctx = gst.pbutils.InstallPluginsContext()
-                    gst.pbutils.install_plugins_async([detail], ctx,
-                                                        self.install_plugin_cb)
+        elif gst.pbutils.is_missing_plugin_message(message):
+            global user_canceled_codec_installation
+            detail = gst.pbutils.missing_plugin_message_get_installer_detail(message)
+            debug('missing plugin:', detail.split('|')[3] , self.sound_file.uri)
+            self.pipeline.set_state(gst.STATE_NULL)
+            if gst.pbutils.install_plugins_installation_in_progress():
+                while gst.pbutils.install_plugins_installation_in_progress():
+                    gtk_sleep(0.1)
+                self.restart()
+                return
+            if user_canceled_codec_installation:
+                self.error = 'Plugin installation cancelled'
+                debug(self.error)
+                self.done()
+                return
+            ctx = gst.pbutils.InstallPluginsContext()
+            gst.pbutils.install_plugins_async([detail], ctx, self.install_plugin_cb)
 
         elif t == gst.MESSAGE_EOS:
             self.eos = True
@@ -328,7 +358,6 @@ class Decoder(Pipeline):
         self.sound_file = sound_file
         self.time = 0
         self.position = 0
-        self.probe_id = None
 
         command = '%s location="%s" name=src ! decodebin name=decoder' % \
             (gstreamer_source, encode_filename(self.sound_file.uri))
@@ -344,15 +373,37 @@ class Decoder(Pipeline):
         pass
 
     def query_duration(self):
+        """
+        Ask for the duration of the current pipeline.
+        """
         try:
             if not self.sound_file.duration and self.pipeline:
                 self.sound_file.duration = self.pipeline.query_duration(
                                             gst.FORMAT_TIME)[0] / gst.SECOND
                 debug('got file duration:', self.sound_file.duration)
+                if self.sound_file.duration < 0:
+                    self.sound_file.duration = None
         except gst.QueryError:
             self.sound_file.duration = None
 
+    def query_position(self):
+        """
+        Ask for the stream position of the current pipeline.
+        """
+        try:
+            if self.pipeline:
+                self.position = self.pipeline.query_position(
+                                            gst.FORMAT_TIME)[0] / gst.SECOND
+                if self.position < 0:
+                    self.position = 0
+        except gst.QueryError:
+            self.position = 0
+
+
     def found_tag(self, decoder, something, taglist):
+        """
+        Called when the decoder reads a tag.
+        """
         debug('found_tags:', self.sound_file.filename_for_display)
         for k in taglist.keys():
             if 'image' not in k:
@@ -380,33 +431,14 @@ class Decoder(Pipeline):
                 tags[k] = taglist[k]
 
         self.sound_file.tags.update(tags)
-
-        try:
-            self.sound_file.duration = self.pipeline.query_duration(
-                                            gst.FORMAT_TIME)[0] / gst.SECOND
-        except gst.QueryError:
-            pass
-
-    def _buffer_probe(self, pad, buffer):
-        """buffer probe callback used to get real time
-           since the beginning of the stream"""
-        if buffer.timestamp == gst.CLOCK_TIME_NONE:
-            pad.remove_buffer_probe(self.probe_id)
-            return False
-
-        self.position = float(buffer.timestamp) / gst.SECOND
-        return True
+        self.query_duration()
 
     def new_decoded_pad(self, decoder, pad, is_last):
         """ called when a decoded pad is created """
-        self.probe_id = pad.add_buffer_probe(self._buffer_probe)
-        self.probed_pad = pad
-        self.processing = True
         self.query_duration()
+        self.processing = True
 
     def finished(self):
-        if self.probe_id:
-            self.probed_pad.remove_buffer_probe(self.probe_id)
         Pipeline.finished(self)
 
     def get_sound_file(self):
@@ -422,12 +454,9 @@ class Decoder(Pipeline):
 
     def get_position(self):
         """ return the current pipeline position in the stream """
+        self.query_position()
         return self.position
 
-# this is needed since TagReader caller don't keep a reference long enough,
-# and so tagReaders will be deleted before the callback are called,
-# resulting in awful crashes...
-dontdelete = []
 
 class TagReader(Decoder):
 
@@ -442,7 +471,6 @@ class TagReader(Decoder):
         self.add_command('fakesink')
         self.add_signal(None, 'message::state-changed', self.on_state_changed)
         self.tagread = False
-        dontdelete.append(self)
 
     def set_found_tag_hook(self, found_tag_hook):
         self.found_tag_hook = found_tag_hook
@@ -459,7 +487,6 @@ class TagReader(Decoder):
         self.sound_file.tags_read = True
         if self.found_tag_hook:
             gobject.idle_add(self.found_tag_hook, self)
-        dontdelete.remove(self)
 
 
 class Converter(Decoder):
@@ -485,7 +512,6 @@ class Converter(Decoder):
         self.resample_rate = resample_rate
         self.force_mono = force_mono
 
-        self.overwrite = False
         self.delete_original = delete_original
 
         self.got_duration = False
@@ -500,15 +526,15 @@ class Converter(Decoder):
             'audio/ogg; codecs=opus': self.add_opus_encoder,
             'gst-profile': self.add_audio_profile,
         }
-        self.add_command('audiorate')
-        self.add_command('audioresample')
+        self.add_command('audiorate tolerance=10000000')
         self.add_command('audioconvert')
+        self.add_command('audioresample')
 
         # audio resampling support
         if self.output_resample:
             self.add_command('audio/x-raw-int,rate=%d' % self.resample_rate)
-            self.add_command('audioresample')
             self.add_command('audioconvert')
+            self.add_command('audioresample')
 
         if self.force_mono:
             self.add_command('audio/x-raw-int,channels=1')
@@ -518,7 +544,7 @@ class Converter(Decoder):
         if not encoder:
             # TODO: is this used ?
             # TODO: add proper error management when an encoder cannot be created
-            show_error(_("Cannot create a decoder for \'%s\' format.") % 
+            show_error(_('Error', "Cannot create a decoder for \'%s\' format.") % 
                         self.output_type)
             return
 
@@ -529,26 +555,23 @@ class Converter(Decoder):
         if dirname and not gnomevfs.exists(dirname):
             log('Creating folder: \'%s\'' % dirname)
             if not vfs_makedirs(str(dirname)):
-                show_error(_("Cannot create \'%s\' folder.") % dirname)
+                show_error('Error', _("Cannot create \'%s\' folder.") % dirname)
                 return
 
         self.add_command('%s location="%s"' % (
             gstreamer_sink, encode_filename(self.output_filename)))
-        if self.overwrite and vfs_exists(self.output_filename):
-            log('overwriting \'%s\'' % self.output_filename)
-            vfs_unlink(self.output_filename)
+
+    def aborted(self):
+        # remove partial file
+        try:
+            gnomevfs.unlink(self.output_filename)
+        except:
+            log('cannot delete: \'%s\'' % beautify_uri(self.output_filename))
+        return
 
     def finished(self):
         Pipeline.finished(self)
 
-        if self.aborted:
-            # remove partial file
-            try:
-                gnomevfs.unlink(self.output_filename)
-            except:
-                pass
-            return
-
         # Copy file permissions
         try:
             info = gnomevfs.get_file_info(self.sound_file.uri,
@@ -562,14 +585,14 @@ class Converter(Decoder):
         if self.delete_original and self.processing and not self.error:
             log('deleting: \'%s\'' % self.sound_file.uri)
             try:
-                gnomevfs.unlink(self.sound_file.uri)
+                vfs_unlink(self.sound_file.uri)
             except:
                 log('Cannot remove \'%s\'' %
                         gnomevfs.format_uri_for_display(self.output_filename))
 
     def on_error(self, err):
         #pass
-
+        self.error = err
         show_error('<b>%s</b>' % _('GStreamer Error:'), '%s\n<i>(%s)</i>' % (err,
                     self.sound_file.filename_for_display))
 
@@ -613,27 +636,16 @@ class Converter(Decoder):
         return cmd
 
     def add_mp3_encoder(self):
-
-        cmd = 'lame quality=2 ' # DEPRECATED !
+        cmd = 'lamemp3enc encoding-engine-quality=2 '
 
         if self.mp3_mode is not None:
             properties = {
-                'cbr' : (0, 'bitrate'),
-                'abr' : (3, 'vbr-mean-bitrate'),
-                'vbr' : (4, 'vbr-quality')
+                'cbr' : 'target=bitrate cbr=true bitrate=%s ',
+                'abr' : 'target=bitrate cbr=false bitrate=%s ',
+                'vbr' : 'target=quality cbr=false quality=%s ',
             }
 
-            cmd += 'vbr=%s ' % properties[self.mp3_mode][0]
-            if self.mp3_quality == 9:
-                # GStreamer set max bitrate to 320 but lame uses
-                # mpeg2 with vbr-quality==9, so max bitrate is 160
-                # - update: now set to 128 since lame don't accept 160 anymore.
-                cmd += 'vbr-max-bitrate=128 '
-            elif properties[self.mp3_mode][0]:
-                cmd += 'vbr-max-bitrate=320 '
-            cmd += '%s=%s ' % (properties[self.mp3_mode][1], self.mp3_quality)
-            if properties[self.mp3_mode][0] and self.mp3_quality < 4:
-                cmd += 'lowpass-freq=20000 '
+            cmd += properties[self.mp3_mode] % self.mp3_quality
 
             if 'xingmux' in available_elements and properties[self.mp3_mode][0]:
                 # add xing header when creating VBR mp3
@@ -656,21 +668,6 @@ class Converter(Decoder):
         return pipeline
 
 
-class ConverterQueueCanceled(SoundConverterException):
-
-    """Exception thrown when a ConverterQueue is canceled."""
-
-    def __init__(self):
-        SoundConverterException.__init__(self, _('Conversion Canceled'), '')
-
-class ConverterQueueError(SoundConverterException):
-
-    """Exception thrown when a ConverterQueue had an error."""
-
-    def __init__(self):
-        SoundConverterException.__init__(self, _('Conversion Error'), '')
-
-
 class ConverterQueue(TaskQueue):
 
     """Background task for converting many files."""
@@ -678,90 +675,25 @@ class ConverterQueue(TaskQueue):
     def __init__(self, window):
         TaskQueue.__init__(self)
         self.window = window
-        self.overwrite_action = None
         self.reset_counters()
 
     def reset_counters(self):
         self.total_duration = 0
         self.duration_processed = 0
-        self.overwrite_action = None
         self.errors = []
         self.error_count = 0
         self.all_tasks = None
+        global user_canceled_codec_installation
+        user_canceled_codec_installation = True
 
     def add(self, sound_file):
-        output_filename = self.window.prefs.generate_filename(sound_file)
-        path = urlparse(output_filename) [2]
-        path = unquote_filename(path)
-
-        exists = True
-        try:
-            gnomevfs.get_file_info(gnomevfs.URI((output_filename)))
-        except gnomevfs.NotFoundError:
-            exists = False
-        except gnomevfs.AccessDeniedError:
-            self.error_count += 1
-            msg = _('Access denied: \'%s\'' % output_filename)
-            log(msg)
-            show_error(msg, '')
-            raise ConverterQueueError()
-            return
-        except:
-            self.error_count += 1
-            msg = 'Invalid URI: \'%s\'' % output_filename
-            log(msg)
-            show_error(msg, '')
-            raise ConverterQueueError()
-            return
-
-        # do not overwrite source file !!
-        if output_filename == sound_file.uri:
-            self.error_count += 1
-            show_error(_('Cannot overwrite source file(s)!'), '')
-            raise ConverterQueueCanceled()
-
-        if exists:
-            if self.overwrite_action is not None:
-                result = self.overwrite_action
-            else:
-                dialog = self.window.existsdialog
-
-                dpath = os.path.basename(path)
-                dpath = gobject.markup_escape_text(dpath)
-
-                msg = \
-                _('The output file <i>%s</i>\n already exists.\n '\
-                    'Do you want to skip the file, overwrite it or'\
-                    ' cancel the conversion?\n') % dpath
-
-                dialog.message.set_markup(msg)
-                dialog.set_transient_for(self.window.widget)
-
-                if self.overwrite_action is not None:
-                    dialog.apply_to_all.set_active(True)
-                else:
-                    dialog.apply_to_all.set_active(False)
-
-                result = dialog.run()
-                dialog.hide()
-
-                if dialog.apply_to_all.get_active():
-                    if result == 1 or result == 0:
-                        self.overwrite_action = result
-
-            if result == 1:
-                # overwrite
-                try:
-                    vfs_unlink(output_filename)
-                except gnomevfs.NotFoundError:
-                    pass
-            elif result == 0:
-                # skip file
-                return
-            else:
-                # cancel operation
-                raise ConverterQueueCanceled()
+        # generate a temporary filename from source name and output suffix
+        output_filename = self.window.prefs.generate_temp_filename(sound_file) + '~SC~'
 
+        if vfs_exists(output_filename):
+            # always overwrite temporary files
+            vfs_unlink(output_filename)
+        
         c = Converter(sound_file, output_filename,
                         self.window.prefs.get_string('output-mime-type'),
                         self.window.prefs.get_int('delete-original'),
@@ -822,14 +754,40 @@ class ConverterQueue(TaskQueue):
         return self.running or len(self.all_tasks), progress
 
     def on_task_finished(self, task):
+        task.sound_file.progress = 1.0
+        
+        if task.error:
+            debug('error in task, skipping rename:', task.output_filename)
+            if vfs_exists(task.output_filename):
+                vfs_unlink(task.output_filename)
+            self.errors.append(task.error)
+            self.error_count += 1
+            return
+            
         duration = task.get_duration()
         if duration:
             self.duration_processed += duration
-        self.errors.append(task.error)
+
+        # rename temporary file 
+        newname = self.window.prefs.generate_filename(task.sound_file)
+        log(beautify_uri(task.output_filename), '->', beautify_uri(newname))
+        
+        # safe mode. generate a filename until we find a free one
+        p,e = os.path.splitext(newname)
+        p = p.replace('%', '%%')
+        p = p + ' (%d)' + e
+        i = 1
+        while vfs_exists(newname):
+            newname = p % i
+            i += 1
+        
+        task.error = vfs_rename(task.output_filename, newname)
         if task.error:
+            self.errors.append(task.error)
             self.error_count += 1
 
     def finished(self):
+        # This must be called with emit_async
         if self.running_tasks:
             raise RuntimeError
         TaskQueue.finished(self)
@@ -863,5 +821,3 @@ class ConverterQueue(TaskQueue):
         TaskQueue.abort(self)
         self.window.set_sensitive()
         self.reset_counters()
-
-
