@@ -19,11 +19,9 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 # USA
 
-import string
 import time
 import os
 import urllib
-import unicodedata
 import gnomevfs
 from fileoperations import vfs_exists
 
@@ -32,7 +30,7 @@ class TargetNameGenerator:
 
     """Generator for creating the target name from an input name."""
 
-    nice_chars = string.ascii_letters + string.digits + '.-_/'
+    bad_chars = u'\\?%*:|"<>\ufffd'
 
     def __init__(self):
         self.folder = None
@@ -44,18 +42,6 @@ class TargetNameGenerator:
         self.max_tries = 2
         self.exists = vfs_exists
 
-    def _unicode_to_ascii(self, unicode_string):
-        # thanks to
-        # http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/251871
-        try:
-            unicode_string = unicode(unicode_string, 'utf-8')
-            return unicodedata.normalize('NFKD', unicode_string).encode(
-                                                            'ASCII', 'ignore')
-        except UnicodeDecodeError:
-            unicode_string = unicode(unicode_string, 'iso-8859-1')
-            return unicodedata.normalize('NFKD', unicode_string).encode(
-                                                            'ASCII', 'replace')
-
     def get_target_name(self, sound_file):
 
         assert self.suffix, 'you just forgot to call set_target_suffix()'
@@ -66,7 +52,7 @@ class TargetNameGenerator:
         root = sound_file.base_path
         basename, ext = os.path.splitext(urllib.unquote(sound_file.filename))
 
-        # make sure basename constains only the filename
+        # make sure basename contains only the filename
         basefolder, basename = os.path.split(basename)
 
         d = {
@@ -97,23 +83,23 @@ class TargetNameGenerator:
 
         pattern = os.path.join(self.subfolders, self.basename + self.suffix)
         result = pattern % d
-        if isinstance(result, unicode):
-            result = result.encode('utf-8')
 
         if self.replace_messy_chars:
-            result = self._unicode_to_ascii(result)
-            s = ''
-            for c in result:
-                if c not in self.nice_chars:
-                    s += '_'
-                else:
-                    s += c
-            result = s
+            # convert to unicode object to manage filename letter by letter
+            if not isinstance(result, unicode):
+                result = unicode(result, 'utf-8', 'replace')
+
+            for char in self.bad_chars:
+                result = result.replace(char, '_')
+
+        # convert back to string so that urllib could cope with it
+        if isinstance(result, unicode):
+            result = result.encode('utf-8')
 
         if self.folder is None:
             folder = root
         else:
-            folder = urllib.quote(self.folder, '/:')
+            folder = urllib.quote(self.folder, '/:%@')
 
         if '/' in pattern:
             # we are creating folders using tags, disable basefolder handling
@@ -122,13 +108,3 @@ class TargetNameGenerator:
         result = os.path.join(folder, basefolder, urllib.quote(result))
 
         return result
-        
-    def require_tags(self):
-        from settings import patterns_formats
-        pattern = os.path.join(self.subfolders, self.basename + self.suffix)
-        for p in patterns_formats:
-            if p in pattern:
-                return True
-        return False        
-        
-        
