@@ -20,6 +20,7 @@
 # USA
 
 import os
+from os.path import basename, dirname
 import time
 import sys
 import gtk
@@ -33,7 +34,7 @@ from gconfstore import GConfStore
 from fileoperations import filename_to_uri, beautify_uri
 from fileoperations import unquote_filename, vfs_walk
 from fileoperations import use_gnomevfs
-from gstreamer import ConverterQueue, ConverterQueueCanceled, ConverterQueueError
+from gstreamer import ConverterQueue
 from gstreamer import available_elements, TypeFinder, TagReader
 from gstreamer import audio_profiles_list, audio_profiles_dict
 from soundfile import SoundFile
@@ -72,16 +73,6 @@ def gtk_sleep(duration):
         gtk_iteration()
 
 
-def format_tag(tag):
-    if isinstance(tag, list):
-        if len(tag) > 1:
-            tag = ', '.join(tag[:-1]) + ' & ' + tag[-1]
-        else:
-            tag = tag[0]
-
-    return tag
-
-
 class ErrorDialog:
 
     def __init__(self, builder):
@@ -93,14 +84,13 @@ class ErrorDialog:
     def show_error(self, primary, secondary):
         self.primary.set_markup(primary)
         self.secondary.set_markup(secondary)
-        sys.stderr.write(_('\nError: %s\n%s\n') % (primary, secondary))
+        try:
+            sys.stderr.write(_('\nError: %s\n%s\n') % (primary, secondary))
+        except:
+            pass
         self.dialog.run()
         self.dialog.hide()
 
-    def show_exception(self, exception):
-        self.show('<b>%s</b>' % gobject.markup_escape_text(exception.primary),
-                    exception.secondary)
-
 
 class MsgAreaErrorDialog_:
 
@@ -109,7 +99,10 @@ class MsgAreaErrorDialog_:
         self.primary = builder.get_object('label_error')
 
     def show_error(self, primary, secondary):
-        sys.stderr.write(_('\nError: %s\n%s\n') % (primary, secondary))
+        try:
+            sys.stderr.write(_('\nError: %s\n%s\n') % (primary, secondary))
+        except:
+            pass
         #self.msg_area.set_text_and_icon(gtk.STOCK_DIALOG_ERROR, primary, secondary)
         #self.msg_area.show()
         self.primary.set_text(primary)
@@ -176,7 +169,7 @@ class FileList:
 
     def drag_data_received(self, widget, context, x, y, selection,
                              mime_id, time):
-
+        widget.stop_emission('drag_data_received')
         if mime_id >= 0 and mime_id < len(self.drop_mime_types):
             self.add_uris([uri.strip() for uri in selection.data.split('\n')])
             context.finish(True, False, time)
@@ -247,9 +240,16 @@ class FileList:
             else:
                 files.append(uri)
 
+        files = [f for f in files if not f.endswith('~SC~')]
+
         if not base:
-            base, notused = os.path.split(os.path.commonprefix(files))
-        base += '/'
+            base = os.path.commonprefix(files)
+            if base and not base.endswith('/'):
+                # we want a common folder
+                base = base[0:base.rfind('/')]
+                base += '/'
+        else:
+            base += '/'
 
         for f in files:
             sound_file = SoundFile(f, base)
@@ -541,14 +541,19 @@ class PreferencesDialog(GladeWindow, GConfStore):
         model = self.output_mime_type.get_model()
         for b in widgets:
             mime, encoder_name = b
-            encoder_present = encoder_name in available_elements
-            if encoder_name and not encoder_present:
+            # valid encoder?
+            encoder_present = encoder_name and encoder_name in available_elements
+            # valid profile?
+            profile_present = mime == 'gst-profile' and audio_profiles_list
+            if encoder_present or profile_present:
+                # add to supported outputs
+                self.present_mime_types.append(mime)
+                i += 1
+            else:
+                # remove it.
                 del model[i]
                 if mime_type == mime:
                     mime_type = self.defaults['output-mime-type']
-            else:
-                self.present_mime_types.append(mime)
-                i += 1
         for i, mime in enumerate(self.present_mime_types):
             if mime_type == mime:
                 widget.set_active(i)
@@ -719,40 +724,38 @@ class PreferencesDialog(GladeWindow, GConfStore):
                     self.get_bitrate_from_settings())
         self.aprox_bitrate.set_markup(markup)
 
-    def generate_filename(self, sound_file, for_display=False):
+    def get_output_suffix(self):
         self.gconf.clear_cache()
         output_type = self.get_string('output-mime-type')
         profile = self.get_string('audio-profile')
         profile_ext = audio_profiles_dict[profile][1] if profile else ''
         output_suffix = {
-                        'audio/x-vorbis': '.ogg',
-                        'audio/x-flac': '.flac',
-                        'audio/x-wav': '.wav',
-                        'audio/mpeg': '.mp3',
-                        'audio/x-m4a': '.m4a',
-                        'audio/ogg; codecs=opus': '.opus',
-                        'gst-profile': '.' + profile_ext,
-                    }.get(output_type, '.?')
-
-        generator = TargetNameGenerator()
-
+                'audio/x-vorbis': '.ogg',
+                'audio/x-flac': '.flac',
+                'audio/x-wav': '.wav',
+                'audio/mpeg': '.mp3',
+                'audio/x-m4a': '.m4a',
+                'audio/ogg; codecs=opus': '.opus',
+                'gst-profile': '.' + profile_ext,
+        }.get(output_type, '.?')
         if output_suffix == '.ogg' and self.get_int('vorbis-oga-extension'):
             output_suffix = '.oga'
+        return output_suffix
 
-        generator.suffix = output_suffix
+    def generate_filename(self, sound_file, for_display=False):
+        generator = TargetNameGenerator()
+        generator.suffix = self.get_output_suffix()
 
         if not self.get_int('same-folder-as-input'):
             folder = self.get_string('selected-folder')
             folder = filename_to_uri(folder)
             generator.folder = folder
 
-        if self.get_int('create-subfolders'):
-            generator.subfolders = self.get_subfolder_pattern()
+            if self.get_int('create-subfolders'):
+                generator.subfolders = self.get_subfolder_pattern()
 
         generator.basename = self.get_basename_pattern()
 
-        self.require_tags = generator.require_tags()
-
         if for_display:
             generator.replace_messy_chars = False
             return unquote_filename(generator.get_target_name(sound_file))
@@ -760,6 +763,13 @@ class PreferencesDialog(GladeWindow, GConfStore):
             generator.replace_messy_chars = self.get_int('replace-messy-chars')
             return generator.get_target_name(sound_file)
 
+    def generate_temp_filename(self, soundfile):
+        folder = dirname(soundfile.uri)
+        if not self.get_int('same-folder-as-input'):
+            folder = self.get_string('selected-folder')
+            folder = filename_to_uri(folder)
+        return folder + '/' + basename(soundfile.filename)
+
     def process_custom_pattern(self, pattern):
         for k in custom_patterns:
             pattern = pattern.replace(k, custom_patterns[k])
@@ -812,9 +822,9 @@ class PreferencesDialog(GladeWindow, GConfStore):
 
     def on_choose_folder_clicked(self, button):
         ret = self.target_folder_chooser.run()
+        folder = self.target_folder_chooser.get_uri()
         self.target_folder_chooser.hide()
         if ret == gtk.RESPONSE_OK:
-            folder = self.target_folder_chooser.get_uri()
             if folder:
                 self.set_string('selected-folder', urllib.unquote(folder))
                 self.update_selected_folder()
@@ -1254,8 +1264,8 @@ class SoundConverterWindow(GladeWindow):
             self.addchooser.set_current_folder_uri(last_folder)
 
         ret = self.addchooser.run()
-        self.addchooser.hide()
         folder = self.addchooser.get_current_folder_uri()
+        self.addchooser.hide()
         if ret == gtk.RESPONSE_OK and folder:
             self.filelist.add_uris(self.addchooser.get_uris())
             self.prefs.set_string('last-used-folder', folder)
@@ -1267,18 +1277,15 @@ class SoundConverterWindow(GladeWindow):
             self.addfolderchooser.set_current_folder_uri(last_folder)
 
         ret = self.addfolderchooser.run()
+        folders = self.addfolderchooser.get_uris()
+        folder = self.addfolderchooser.get_current_folder_uri()
         self.addfolderchooser.hide()
         if ret == gtk.RESPONSE_OK:
-
-            folders = self.addfolderchooser.get_uris()
-
             extensions = None
             if self.combo.get_active():
                 patterns = filepattern[self.combo.get_active()][1].split(';')
                 extensions = [os.path.splitext(p)[1] for p in patterns]
             self.filelist.add_uris(folders, extensions=extensions)
-
-            folder = self.addfolderchooser.get_current_folder_uri()
             if folder:
                 self.prefs.set_string('last-used-folder', folder)
 
@@ -1287,9 +1294,15 @@ class SoundConverterWindow(GladeWindow):
     def on_remove_activate(self, *args):
         model, paths = self.filelist_selection.get_selected_rows()
         while paths:
-            i = self.filelist.model.get_iter(paths[0])
+            # Remove files
+            childpath = model.convert_path_to_child_path(paths[0])
+            i = self.filelist.model.get_iter(childpath)
             self.filelist.remove(i)
             model, paths = self.filelist_selection.get_selected_rows()
+        # re-assign row numbers
+        files = self.filelist.get_files()
+        for i, sound_file in enumerate(files):
+            sound_file.filelist_row = i
         self.set_sensitive()
 
     def on_clearlist_activate(self, *args):
@@ -1298,26 +1311,15 @@ class SoundConverterWindow(GladeWindow):
         self.set_sensitive()
         self.set_status()
 
-    def read_tags(self, sound_file):
-        if sound_file.tags_read: # TODO: factorize
-            self.converter.add(sound_file)
-            return
-
-        tagreader = TagReader(sound_file)
-        tagreader.set_found_tag_hook(self.tags_read)
-        tagreader.start()
-
-    def tags_read(self, tagreader):
-        sound_file = tagreader.get_sound_file()
-        self.converter.add(sound_file)
-
     def on_progress(self):
-        if self.pulse_progress >= 0: # still waiting for tags
+        if self.pulse_progress > 0: # still waiting for tags
             self.set_progress(self.pulse_progress, display_time=False)
             return True
         if self.pulse_progress == -1: # still waiting for add
             self.set_progress()
             return True
+        if self.pulse_progress == False: # conversion ended
+            return False
 
         perfile = {}
         for s in self.filelist.get_files():
@@ -1336,37 +1338,21 @@ class SoundConverterWindow(GladeWindow):
         return running
 
     def do_convert(self):
-        try:
-            self.pulse_progress = -1
-            gobject.timeout_add(100, self.on_progress)
-            if self.prefs.require_tags:
-                self.progressbar.set_text(_('Reading tags...'))
-            else:
-                self.progressbar.set_text(_('Preparing conversion...'))
-            files = self.filelist.get_files()
-            total = len(files)
-            for i, sound_file in enumerate(files):
-                gtk_iteration()
-                self.pulse_progress = float(i)/total
-                sound_file.progress = None
-                if self.prefs.require_tags:
-                    self.read_tags(sound_file)
-                else:
-                    self.converter.add(sound_file)
-
-        except ConverterQueueCanceled:
-            log('cancelling conversion.')
-            self.conversion_ended()
-            self.set_status(_('Conversion cancelled'))
-        except ConverterQueueError:
-            self.conversion_ended()
-            self.set_status(_('Error when converting'))
-        else:
-            self.set_status('')
-            self.pulse_progress = None
-            self.converter.start()
-            self.set_sensitive()
-        return False
+        self.pulse_progress = -1
+        gobject.timeout_add(100, self.on_progress)
+        self.progressbar.set_text(_('Preparing conversion...'))
+        files = self.filelist.get_files()
+        total = len(files)
+        for i, sound_file in enumerate(files):
+            gtk_iteration()
+            self.pulse_progress = float(i)/total # TODO: still needed?
+            sound_file.progress = None
+            self.converter.add(sound_file)
+        # all was OK
+        self.set_status('')
+        self.pulse_progress = None
+        self.converter.start()
+        self.set_sensitive()
 
     def on_convert_button_clicked(self, *args):
         # reset and show progress bar
@@ -1376,18 +1362,16 @@ class SoundConverterWindow(GladeWindow):
         self.progress_time = time.time()
         self.set_progress()
         self.set_status(_('Converting'))
+        for soundfile in self.filelist.get_files():
+            self.set_file_progress(soundfile, 0.0)
         # start conversion
         self.do_convert()
         # update ui
         self.set_sensitive()
 
     def on_button_pause_clicked(self, *args):
-        tasks = self.converter.running_tasks
-        if not tasks:
-            return
-        self.converter.paused = not self.converter.paused
-        for task in tasks:
-            task.toggle_pause(self.converter.paused)
+        self.converter.toggle_pause(not self.converter.paused)
+            
         if self.converter.paused:
             self.current_pause_start = time.time()
         else:
@@ -1508,6 +1492,7 @@ def gui_main(name, version, gladefile, input_files):
     NAME, VERSION = name, version
     gnome.init(name, version)
     builder = gtk.Builder()
+    builder.set_translation_domain(name.lower())
     builder.add_from_file(gladefile)
 
     global win
